###############################################################################
#                                                                             #
#                     Change Mailbox Password for Office 365                  #
#                                    V2.3                                     #
#                       (C)2006-2012 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)]: $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($UserID)
$S="ErrorsCount[$($UserID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Changing user's password for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2012 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
function proxy
{
Param([string] $Proxy,$User,$Password)
if (![String]::IsNullorEmpty($Proxy))
   {
   if (![String]::IsNullorEmpty($User))
      { 
      $SecuredPassword=ConvertTo-SecureString $Password -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $User, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection through proxy $($Proxy), User=$($User)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection through proxy $($Proxy)"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword)
$error.Clear()
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
$Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function user_exists
{
Param([string] $identity)
$error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $identity {Param($id) get-user -Identity $id}
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function change_user_password
{
Param($UserID,$UserPassword,$UserDisplayName)
$error.Clear()
# Secure the password
if (![String]::IsNullorEmpty($UserPassword))
   {
	$SecuredPassword = ConvertTo-SecureString $UserPassword -AsPlainText -Force
   }
   else
   {
	$SecuredPassword = $UserPassword
   }
%{Invoke-Command -Session $Script:RS {param ($id,$pwd) set-mailbox -Identity $id -Password $pwd}-arg $UserID,$SecuredPassword}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully change password for $($UserDisplayName) ($($UserID)) ($($UserEmail))"
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Changing password for $($UserDisplayName) ($($UserID)) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }
}
#
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V2.3"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword="%OFFICE365_PASSWORD%"
#
# Proxy parameters
#
$ProxyAddress="%PROXY_ADDRESS%"
$ProxyUser="%PROXY_USER%"     
$ProxyPassword="%PROXY_PASSWORD%"
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
proxy $ProxyAddress $ProxyUser $ProxyPassword
connect $Office365Url $Office365LoginName $Office365LoginPassword
if ($global:total_errors_count -eq 0)
{
### BEGIN_REPETITION ###
$global:errors_count=0
# KoXo Administrator's variables
$UserID="%USER_ID%"
$UserPassword="%USER_PASSWORD%"
$UserDisplayName="%USER_TITLE% %USER_FIRST_NAME% %USER_LAST_NAME%"
$UserDisplayName=$UserDisplayName.Trim()
$UserEmail="%USER_EMAIL%"
log_message "Office 365" "$($UserDisplayName) ($($UserEmail))"
if (user_exists($UserID))
   {
   write-host "User : $($UserDisplayName) ($($UserID)) ($($UserEmail))" 
   change_user_password $UserID $UserPassword $UserDisplayName
   }
   else
   {
   $errors_count++
   $S="User ""$($UserID)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserID)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
### END_REPETITION ###
close
}
log_total_errors_count
